#include "FAtoLRGConverter.h"

using namespace std;
using namespace automaton;
using namespace grammar;
using namespace alib;

namespace conversions
{

FAtoLRGConverter::FAtoLRGConverter( const FSM & automaton ) : m_automaton( automaton )
{
    if( ! automaton.isEpsilonFree( ) )
        throw AlibException( "FSM has epsilon transitions." );
}

FAtoLRGConverter::~FAtoLRGConverter( void )
{

}

LeftRegularGrammar FAtoLRGConverter::convert( void )
{
    LeftRegularGrammar grammar;
    map<State, Symbol> nonterminalMap;

    for( const auto & symbol : m_automaton.getInputAlphabet( ) )
        grammar.addTerminalSymbol( symbol );

    for( const auto & state : m_automaton.getStates( ) )
    {
        Symbol nt = grammar.createUniqueNonTerminalSymbol( state.getName( ) );
        nonterminalMap.insert( pair<State,Symbol>( state, nt ) );
    }

    // step 2 - create set of P in G
    for( const auto & transition : m_automaton.getTransitions( ) )
    {
        // 2a
        list<Symbol> leftSide, rightSide;
        leftSide.push_back( nonterminalMap.find( transition.getTo( ) )->second );
        rightSide.push_back( nonterminalMap.find( transition.getFrom( ) )->second );
        rightSide.push_back( transition.getInput( ) );
        grammar.addRule( Rule( leftSide, rightSide ) );

        // 2b
        if( isInSet( transition.getFrom( ), m_automaton.getInitialStates( ) ) )
        {
            list<Symbol> leftSide, rightSide;
            leftSide.push_back( nonterminalMap.find( transition.getTo( ) )->second );
            rightSide.push_back( transition.getInput( ) );
            grammar.addRule( Rule( leftSide, rightSide ) );
        }
    }

    // step 3
    Symbol newStart = grammar.createUniqueNonTerminalSymbol( "S", false );
    grammar.setStartSymbol( newStart );

    //
    if( isInSet( * m_automaton.getInitialStates( ).begin( ), m_automaton.getFinalStates( ) ) )
    {
        list<Symbol> leftSide = { newStart }, rightSide;
        grammar.addRule( Rule( leftSide, rightSide ) );
    }

    // pro vsecky neterminaly, ktere vznikly z finalnich stavu:
        // kopiruj jejich prave strany k newStart
        // pokud nejsou nikde vpravo, smaz je

    for( const auto & rule : grammar.getRules( ) )
    {
        const Symbol & leftSide = rule.getLeftSide( ).front( );

        if( isInSet( State( leftSide.getSymbol( ) ), m_automaton.getFinalStates( ) ) )
        {
            list<Symbol> leftSide = { newStart }, rightSide( rule.getRightSide( ) );
            grammar.addRule( Rule( leftSide, rightSide ) );
        }

        // jetenhle krok nutny,???
        // if( leftSide != grammar.getStartSymbol( ) && ! grammar.isNonTerminalOnRightSideOfAnyRule( leftSide ) )
        //    grammar.removeRule( rule );
    }

    return grammar;
}

} /* namespace conversions */
