/*
 * Glushkov.h
 *
 *  Created on: 11. 1. 2014
 *      Author: tomas
 */

#ifndef GLUSHKOV_H_
#define GLUSHKOV_H_

#include <map>

#include <alphabet/Symbol.h>
#include <automaton/State.h>
#include <regexp/RegExp.h>
#include <regexp/RegExpElements.h>

#include "../interface/IConversionFSM.h"
#include "../shared/glushkov/GlushkovTraversal.h"

namespace conversions
{

/**
 * Converts regular expression to finite automata using Glushkov's NFA construction algorithm.
 * Source: Melichar 2.107
 */
class Glushkov : public IConversionFSM
{
public:
    /**
     * @param re Original regular expression.
     */
    Glushkov( const regexp::RegExp & re );

    ~Glushkov( void );

    /**
     * Performs conversion.
     * @return FSM equivalent to original regular expression.
     *
     */
    automaton::FSM convert( void );

private:
    std::map<GlushkovSymbol, automaton::State> m_stateMap;
    std::set<GlushkovSymbol> m_first, m_last;
    std::set<GlushkovPair> m_pairs;

    /**
     * input regexp
     */
    const regexp::RegExp & m_re;

};

} /* namespace conversions */

#endif /* GLUSHKOV_H_ */
